/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ListIterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author stefan
 */
class SlideImage {
    BufferedImage image;
    byte[]        imageData;
    String        fileName;
    Rectangle     area;
    boolean       load_error;
    Vector        cache;

    SlideImage (File file) {
        try {
            fileName = file.getName();
            loadStream (new FileInputStream (file));
	    area = new Rectangle (0, 0, image.getWidth(), image.getHeight());
	    load_error = false;
        } catch (Exception e) {
            load_error = true;
        }
	cache = new Vector();
    }
    SlideImage()
    {
	image = null;
	cache = new Vector();
    }
    void crop (int x, int y, int w, int h)
    {
        area = new Rectangle (x, y, w, h);
    }
    String cacheId (Rectangle drect)
    {
	return area.toString() + "-" + drect.toString();
    }
    SlideImageBackingStoreEntry lookupBackingStore (String cacheId)
    {
	ListIterator iter = cache.listIterator();

	while (iter.hasNext()) {
	    SlideImageBackingStoreEntry entry = (SlideImageBackingStoreEntry) iter.next();
	    if (entry.cache_id.equals (cacheId))
		return entry;
	}
	return null;

    }
    void draw(Graphics2D g2, Rectangle drect, boolean draw_selection)
    {
	SlideImageBackingStoreEntry backing_store = lookupBackingStore (cacheId (drect));
	if (backing_store == null)
	{
	    Image croppedImage = image.getSubimage(area.x, area.y, area.width, area.height);
	    double scalex, scaley;
	    scalex = (double) drect.width / croppedImage.getWidth(null);
	    scaley = (double) drect.height / croppedImage.getHeight(null);
	    double x = drect.x, y = drect.y;
	    if (scalex > scaley)
	    {
		x += (scalex - scaley) * croppedImage.getWidth(null) / 2;
		scalex = scaley;
	    }
	    else if (scaley > scalex)
	    {
		y += (scaley - scalex) * croppedImage.getHeight(null) / 2;
		scaley = scalex;
	    }
	    AffineTransform t = new AffineTransform();
	    t.scale(scalex, scaley);
	    backing_store = new SlideImageBackingStoreEntry();
	    backing_store.image = new BufferedImage ((int) (croppedImage.getWidth (null) * scalex),
						     (int) (croppedImage.getHeight (null) * scaley),
						     BufferedImage.TYPE_3BYTE_BGR);
	    Graphics2D g3 = backing_store.image.createGraphics();
	    g3.drawImage (croppedImage, t, null);
	    backing_store.dx = (int) x;
	    backing_store.dy = (int) y;
	    backing_store.cache_id = cacheId (drect);
	    cache.add (backing_store);

	    if (cache.size() > 2)
		cache.remove(0);
	}
	AffineTransform t = new AffineTransform();
	t.translate(backing_store.dx, backing_store.dy);
	g2.drawImage(backing_store.image, t, null);
    }
    void save (XMLStreamWriter xtw) throws XMLStreamException
    {
        xtw.writeStartElement("background_image");
        xtw.writeAttribute ("file_name", fileName);
        xtw.writeAttribute("x", Integer.toString (area.x));
        xtw.writeAttribute("y", Integer.toString (area.y));
        xtw.writeAttribute("width", Integer.toString (area.width));
        xtw.writeAttribute("height", Integer.toString (area.height));
        xtw.writeEndElement();
    }
    void load (XMLStreamReader r) throws XMLStreamException
    {
	if (area == null)
	    area = new Rectangle();
        for (int i = 0; i < r.getAttributeCount(); i++)
        {
            String attr_name = r.getAttributeLocalName (i);
            String attr_value = r.getAttributeValue (i);
            if (attr_name.equals ("file_name"))
                fileName = attr_value;
            if (attr_name.equals ("x"))
                area.x = Integer.parseInt (attr_value);
            if (attr_name.equals ("y"))
                area.y = Integer.parseInt (attr_value);
	    if (attr_name.equals ("width"))
		area.width = Integer.parseInt (attr_value);
	    if (attr_name.equals ("height"))
		area.height = Integer.parseInt (attr_value);
        }
        while (r.hasNext())
        {
            int event = r.next();
            if (event == XMLStreamConstants.END_ELEMENT)
                if (r.getLocalName().equals ("background_image"))
                    return;
        }
    }
    void loadFromZip (InputStream inputStream)
    {
        try {
            loadStream (inputStream);
        } catch (IOException ex) {
            Logger.getLogger(SlideImage.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    void loadStream (InputStream inputStream) throws IOException
    {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int n;
        while ((n = inputStream.read (buffer)) > 0)
            outStream.write (buffer, 0, n);
        imageData = outStream.toByteArray();
        image = ImageIO.read (new ByteArrayInputStream (imageData));
    }
}
